@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Reto Buerki
+ * Copyright (C) 2012-2014 Reto Buerki
  * Copyright (C) 2012 Adrian-Ken Rueegsegger
  * Hochschule fuer Technik Rapperswil
  *
@@ -56,6 +56,11 @@ struct sad_entry_t {
 	 */
 	esa_id_type esa_id;
 
+	/**
+	 * Reqid.
+	 */
+	u_int32_t reqid;
+
 	/**
 	 * Source address of CHILD SA.
 	 */
@@ -108,6 +113,19 @@ static bool sad_entry_match(sad_entry_t * const entry, const host_t * const src,
 		   entry->spi == *spi && entry->proto == *proto;
 }
 
+/**
+ * Find a list entry with given reqid, spi and proto values.
+ */
+static bool sad_entry_match_dst(sad_entry_t * const entry,
+								const u_int32_t * const reqid,
+								const u_int32_t * const spi,
+								const u_int8_t * const proto)
+{
+	return entry->reqid == *reqid &&
+		   entry->spi   == *spi &&
+		   entry->proto == *proto;
+}
+
 /**
  * Compare two SAD entries for equality.
  */
@@ -119,6 +137,7 @@ static bool sad_entry_equal(sad_entry_t * const left, sad_entry_t * const right)
 		return FALSE;
 	}
 	return left->esa_id == right->esa_id &&
+		   left->reqid == right->reqid &&
 		   left->src->ip_equals(left->src, right->src) &&
 		   left->dst->ip_equals(left->dst, right->dst) &&
 		   left->spi == right->spi && left->proto == right->proto;
@@ -126,14 +145,15 @@ static bool sad_entry_equal(sad_entry_t * const left, sad_entry_t * const right)
 
 METHOD(tkm_kernel_sad_t, insert, bool,
 	private_tkm_kernel_sad_t * const this, const esa_id_type esa_id,
-	const host_t * const src, const host_t * const dst, const u_int32_t spi,
-	const u_int8_t proto)
+	const u_int32_t reqid, const host_t * const src, const host_t * const dst,
+	const u_int32_t spi, const u_int8_t proto)
 {
 	status_t result;
 	sad_entry_t *new_entry;
 
 	INIT(new_entry,
 		 .esa_id = esa_id,
+		 .reqid = reqid,
 		 .src = (host_t *)src,
 		 .dst = (host_t *)dst,
 		 .spi = spi,
@@ -146,8 +166,9 @@ METHOD(tkm_kernel_sad_t, insert, bool,
 									new_entry);
 	if (result == NOT_FOUND)
 	{
-		DBG3(DBG_KNL, "inserting SAD entry (esa: %llu, src: %H, dst: %H, "
-			 "spi: %x, proto: %u)", esa_id, src, dst, ntohl(spi), proto);
+		DBG3(DBG_KNL, "inserting SAD entry (esa: %llu, reqid: %u, src: %H, "
+			 "dst: %H, spi: %x, proto: %u)", esa_id, reqid, src, dst,
+			 ntohl(spi), proto);
 		new_entry->src = src->clone((host_t *)src);
 		new_entry->dst = dst->clone((host_t *)dst);
 		this->data->insert_last(this->data, new_entry);
@@ -176,18 +197,44 @@ METHOD(tkm_kernel_sad_t, get_esa_id, esa_id_type,
 	if (res == SUCCESS && entry)
 	{
 		id = entry->esa_id;
-		DBG3(DBG_KNL, "getting ESA id of SAD entry (esa: %llu, src: %H, "
-			 "dst: %H, spi: %x, proto: %u)", id, src, dst, ntohl(spi),
-			 proto);
+		DBG3(DBG_KNL, "returning ESA id %llu of SAD entry (src: %H, dst: %H, "
+			 "spi: %x, proto: %u)", id, src, dst, ntohl(spi), proto);
 	}
 	else
 	{
-		DBG3(DBG_KNL, "no SAD entry found");
+		DBG3(DBG_KNL, "no SAD entry found for src %H, dst %H, spi %x, proto %u",
+			 src, dst, ntohl(spi), proto);
 	}
 	this->mutex->unlock(this->mutex);
 	return id;
 }
 
+METHOD(tkm_kernel_sad_t, get_dst_host, host_t *,
+	private_tkm_kernel_sad_t * const this, const u_int32_t reqid,
+	const u_int32_t spi, const u_int8_t proto)
+{
+	host_t *dst = NULL;
+	sad_entry_t *entry = NULL;
+
+	this->mutex->lock(this->mutex);
+	const status_t res = this->data->find_first(this->data,
+												(linked_list_match_t)sad_entry_match_dst,
+												(void**)&entry, &reqid, &spi, &proto);
+	if (res == SUCCESS && entry)
+	{
+		dst = entry->dst;
+		DBG3(DBG_KNL, "returning destination host %H of SAD entry (reqid: %u,"
+			 " spi: %x, proto: %u)", dst, reqid, ntohl(spi), proto);
+	}
+	else
+	{
+		DBG3(DBG_KNL, "no SAD entry found for reqid %u, spi %x, proto: %u",
+			 reqid, ntohl(spi), proto);
+	}
+	this->mutex->unlock(this->mutex);
+	return dst;
+}
+
 METHOD(tkm_kernel_sad_t, _remove, bool,
 	private_tkm_kernel_sad_t * const this, const esa_id_type esa_id)
 {
@@ -242,6 +289,7 @@ tkm_kernel_sad_t *tkm_kernel_sad_create()
 		.public = {
 			.insert = _insert,
 			.get_esa_id = _get_esa_id,
+			.get_dst_host = _get_dst_host,
 			.remove = __remove,
 			.destroy = _destroy,
 		},
